*
       PROGRAM FF2CINT
       INTEGER IARR(8)
       IARR(1) = 1
       IARR(2) = -1
       IARR(3) = -1
       IARR(4) = -1
       IARR(5) = -1
       IARR(6) = -1
       IARR(7) = -1
       IARR(8) = -1
       CALL C2FINT(IARR)
       STOP
       END
