#ifndef ATLAS_CBLASSALIAS_H
   #define ATLAS_CBLASSALIAS_H

#define cblas_asum   cblas_sasum
#define cblas_nrm2   cblas_snrm2
#define cblas_dot    cblas_sdot
#define cblas_axpy   cblas_saxpy
#define cblas_copy   cblas_scopy
#define cblas_scal   cblas_sscal
#define cblas_swap   cblas_sswap
#define cblas_rotm   cblas_srotm
#define cblas_rot    cblas_srot
#define cblas_rotmg  cblas_srotmg
#define cblas_rotg   cblas_srotg
#define cblas_spr2   cblas_sspr2
#define cblas_syr2   cblas_ssyr2
#define cblas_spr    cblas_sspr
#define cblas_syr    cblas_ssyr
#define cblas_ger    cblas_sger
#define cblas_tpsv   cblas_stpsv
#define cblas_tbsv   cblas_stbsv
#define cblas_trsv   cblas_strsv
#define cblas_tpmv   cblas_stpmv
#define cblas_tbmv   cblas_stbmv
#define cblas_trmv   cblas_strmv
#define cblas_spmv   cblas_sspmv
#define cblas_sbmv   cblas_ssbmv
#define cblas_symv   cblas_ssymv
#define cblas_gbmv   cblas_sgbmv
#define cblas_gemv   cblas_sgemv
#define cblas_trsm   cblas_strsm
#define cblas_trmm   cblas_strmm
#define cblas_syr2k  cblas_ssyr2k
#define cblas_syrk   cblas_ssyrk
#define cblas_symm   cblas_ssymm
#define cblas_gemm   cblas_sgemm
#define cblas_iamax  cblas_isamax

#endif
