    /*        Fast GEMM routine for Alpha 21164/21264      */
    /*         on  Linux, Digital UNIX                     */
    /*        by Kazushige Goto <goto@statabo.rim.or.jp>   */

#ifndef VERSION_H
#define VERSION_H

/*
 * Modified by R. Clint Whaley on 11/18/01 to put entire version string on
 * one line, to get around DEC cc compiler bug
 */
#define VERSION "   Fast GEMM routine for Alpha 21164/21264\n       on  Linux, Digital UNIX\n                          date : 2000.02.25\n     by Kazushige Goto <goto@statabo.rim.or.jp>"
#endif
