#ifndef ATLAS_CBLASDALIAS_H
   #define ATLAS_CBLASDALIAS_H

#define cblas_asum   cblas_dasum
#define cblas_nrm2   cblas_dnrm2
#define cblas_dot    cblas_ddot
#define cblas_axpy   cblas_daxpy
#define cblas_copy   cblas_dcopy
#define cblas_scal   cblas_dscal
#define cblas_swap   cblas_dswap
#define cblas_rotm   cblas_drotm
#define cblas_rot    cblas_drot
#define cblas_rotmg  cblas_drotmg
#define cblas_rotg   cblas_drotg
#define cblas_spr2   cblas_dspr2
#define cblas_syr2   cblas_dsyr2
#define cblas_spr    cblas_dspr
#define cblas_syr    cblas_dsyr
#define cblas_ger    cblas_dger
#define cblas_tpsv   cblas_dtpsv
#define cblas_tbsv   cblas_dtbsv
#define cblas_trsv   cblas_dtrsv
#define cblas_tpmv   cblas_dtpmv
#define cblas_tbmv   cblas_dtbmv
#define cblas_trmv   cblas_dtrmv
#define cblas_spmv   cblas_dspmv
#define cblas_sbmv   cblas_dsbmv
#define cblas_symv   cblas_dsymv
#define cblas_gbmv   cblas_dgbmv
#define cblas_gemv   cblas_dgemv
#define cblas_trsm   cblas_dtrsm
#define cblas_trmm   cblas_dtrmm
#define cblas_syr2k  cblas_dsyr2k
#define cblas_syrk   cblas_dsyrk
#define cblas_symm   cblas_dsymm
#define cblas_gemm   cblas_dgemm
#define cblas_iamax  cblas_idamax

#endif
