#ifndef ATLAS_CBLASCALIAS_H
   #define ATLAS_CBLASCALIAS_H

#define cblas_dotc_sub cblas_cdotc_sub
#define cblas_dotu_sub cblas_cdotu_sub
#define cblas_axpy     cblas_caxpy
#define cblas_copy     cblas_ccopy
#define cblas_scal     cblas_cscal
#define cblas_swap     cblas_cswap
#define cblas_hpr2     cblas_chpr2
#define cblas_her2     cblas_cher2
#define cblas_hpr      cblas_chpr
#define cblas_her      cblas_cher
#define cblas_gerc     cblas_cgerc
#define cblas_geru     cblas_cgeru
#define cblas_tpsv     cblas_ctpsv
#define cblas_tbsv     cblas_ctbsv
#define cblas_trsv     cblas_ctrsv
#define cblas_tpmv     cblas_ctpmv
#define cblas_tbmv     cblas_ctbmv
#define cblas_trmv     cblas_ctrmv
#define cblas_hpmv     cblas_chpmv
#define cblas_hbmv     cblas_chbmv
#define cblas_hemv     cblas_chemv
#define cblas_gbmv     cblas_cgbmv
#define cblas_gemv     cblas_cgemv
#define cblas_trsm     cblas_ctrsm
#define cblas_trmm     cblas_ctrmm
#define cblas_her2k    cblas_cher2k
#define cblas_syr2k    cblas_csyr2k
#define cblas_herk     cblas_cherk
#define cblas_syrk     cblas_csyrk
#define cblas_hemm     cblas_chemm
#define cblas_symm     cblas_csymm
#define cblas_gemm     cblas_cgemm
#define cblas_iamax    cblas_icamax
#define cblas_nrm2     cblas_scnrm2
#define cblas_asum     cblas_scasum

#endif
