/*
 *             Automatically Tuned Linear Algebra Software v3.6.0
 *                    (C) Copyright 1997 R. Clint Whaley
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions, and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *   3. The name of the ATLAS group or the names of its contributers may
 *      not be used to endorse or promote products derived from this
 *      software without specific written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ATLAS GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */
#include "atlas_misc.h"
#include "atlas_lvl3.h"

void Mjoin(Mjoin(PATL,putblk),BNM)(int M, int N, TYPE *V, TYPE *C, int ldc,
                                  const SCALAR beta0)
{
   TYPE *st0 = V + (M*N SHIFT), *st1 = V + (M SHIFT);
#if defined(TREAL) && !defined(BETA0) && !defined(BETA1) && !defined(BETAN1)
      const register TYPE beta = beta0;
#endif
#ifdef TCPLX
   TYPE tmp;
   #ifdef BETAXI0
      const register TYPE beta = *beta0;
   #endif

   ldc <<= 1;
   M <<= 1;
#endif
   ldc -= M;
   do
   {
      #if defined(BETA0)
         do *C++ = *V; while (++V != st1);
      #elif defined(BETA1)
         do *C++ += *V; while (++V != st1);
      #else
         do
         {
            #if defined BETAN1
               *C = *V++ - *C;
               C++;
            #else
               #if defined(TREAL) || defined(BETAXi0)
                  *C = beta * *C + *V++;
                  C++;
               #else
                  CMULT2(V, beta0, C, tmp);
                  C += 2;
                  V += 2;
               #endif
            #endif
         }
         while(V != st1);
      #endif
      st1 += M;
      C += ldc;
   }
   while(V != st0);
}
